@@ -37,7 +37,6 @@
 #include "include/fileops.c"
 #include "include/ieee80211.h"
 #include "include/byteops.c"
-
 /*===========================================================================*/
 /* global var */
 
@@ -46,6 +45,8 @@ static int ouicount;
 static int ouilistcount;
 static ouilist_t *ouilist;
 static hashlist_t *hashlist;
+static long int pbkdf2count;
+static long int pbkdf2readerrorcount;
 static long int hashlistcount;
 static long int readcount;
 static long int readerrorcount;
@@ -69,6 +70,8 @@ static int filteressidpartlen;
 static char *filteressidpartptr;
 
 static char *filtervendorptr;
+static char *filtervendorapptr;
+static char *filtervendorclientptr;
 
 static bool flagpsk;
 static bool flagpmk;
@@ -135,7 +138,7 @@ return true;
 static char *getvendor(uint8_t *mac)
 {
 static ouilist_t * zeiger;
-static char *unknown = "unknown";
+static char *unknown = "Unknown";
 
 for(zeiger = ouilist; zeiger < ouilist +ouicount; zeiger++)
 	{
@@ -149,55 +152,58 @@ static void printstatus()
 {
 static char *vendor;
 
-
-printf("\nOUI information file...: %s\n", usedoui);
-if(ouicount > 0)		printf("OUI entires............: %d\n", ouicount);
-if(readcount > 0)		printf("total lines read.......: %ld\n", readcount);
+printf("\nOUI information file..........: %s\n", usedoui);
+if(ouicount > 0)		printf("OUI entires...................: %d\n", ouicount);
+if(readcount > 0)		printf("total lines read..............: %ld\n", readcount);
 if(flagvendorout == true)
 	{
 	printf("\n");
 	return;
 	}
-if(readerrorcount > 0)			printf("read errors............: %ld\n", readerrorcount);
-if(pmkideapolcount > 0)			printf("valid hash lines.......: %ld\n", pmkideapolcount);
-if(pmkidcount > 0)			printf("PMKID hash lines.......: %ld\n", pmkidcount);
-if(eapolcount > 0)			printf("EAPOL hash lines.......: %ld\n", eapolcount);
-if(essidlenmin != 0)			printf("filter by ESSID len min: %d\n", essidlenmin);
-if(essidlenmax != 32)			printf("filter by ESSID len max: %d\n", essidlenmax);
-if(filteressidptr != NULL)		printf("filter by ESSID........: %s\n", filteressidptr);
-if(filteressidpartptr != NULL)		printf("filter by part of ESSID: %s\n", filteressidpartptr);
+if(pbkdf2count > 0)			printf("PBKDF2 lines..................: %ld\n", pbkdf2count);
+if(pbkdf2readerrorcount > 0)		printf("PBKDF2 errors.................: %ld\n", pbkdf2readerrorcount);
+if(readerrorcount > 0)			printf("read errors...................: %ld\n", readerrorcount);
+if(pmkideapolcount > 0)			printf("valid hash lines..............: %ld\n", pmkideapolcount);
+if(pmkidcount > 0)			printf("PMKID hash lines..............: %ld\n", pmkidcount);
+if(eapolcount > 0)			printf("EAPOL hash lines..............: %ld\n", eapolcount);
+if(essidlenmin != 0)			printf("filter by ESSID len min.......: %d\n", essidlenmin);
+if(essidlenmax != 32)			printf("filter by ESSID len max.......: %d\n", essidlenmax);
+if(filteressidptr != NULL)		printf("filter by ESSID...............: %s\n", filteressidptr);
+if(filteressidpartptr != NULL)		printf("filter by part of ESSID.......: %s\n", filteressidpartptr);
 if(flagfiltermacap == true)
 	{
 	vendor = getvendor(filtermacap);
-	printf("filter by MAC..........: %02x%02x%02x%02x%02x%02x (%s)\n", filtermacap[0], filtermacap[1], filtermacap[2], filtermacap[3], filtermacap[4], filtermacap[5], vendor);
+	printf("filter by MAC.................: %02x%02x%02x%02x%02x%02x (%s)\n", filtermacap[0], filtermacap[1], filtermacap[2], filtermacap[3], filtermacap[4], filtermacap[5], vendor);
 	}
 if(flagfiltermacclient == true)
 	{
 	vendor = getvendor(filtermacclient);
-	printf("filter by MAC..........: %02x%02x%02x%02x%02x%02x (%s)\n", filtermacclient[0], filtermacclient[1], filtermacclient[2], filtermacclient[3], filtermacclient[4], filtermacclient[5], vendor);
+	printf("filter by MAC.................: %02x%02x%02x%02x%02x%02x (%s)\n", filtermacclient[0], filtermacclient[1], filtermacclient[2], filtermacclient[3], filtermacclient[4], filtermacclient[5], vendor);
 	}
 
 if(flagfilterouiap == true)
 	{
 	vendor = getvendor(filterouiap);
-	printf("filter AP by OUI.......: %02x%02x%02x (%s)\n", filterouiap[0], filterouiap[1], filterouiap[2], vendor);
+	printf("filter AP by OUI..............: %02x%02x%02x (%s)\n", filterouiap[0], filterouiap[1], filterouiap[2], vendor);
 	}
-if(filtervendorptr != NULL)		printf("filter AP by VENDOR....: %s\n", filtervendorptr);
+if(filtervendorptr != NULL)		printf("filter AP and CLIENT by VENDOR: %s\n", filtervendorptr);
+if(filtervendorapptr != NULL)		printf("filter AP by VENDOR...........: %s\n", filtervendorapptr);
+if(filtervendorclientptr != NULL)	printf("filter CLIENT by VENDOR.......: %s\n", filtervendorclientptr);
 if(flagfilterouiclient == true)
 	{
 	vendor = getvendor(filterouiclient);
-	printf("filter CLIENT by OUI...: %02x%02x%02x (%s)\n", filterouiclient[0], filterouiclient[1], filterouiclient[2], vendor);
+	printf("filter CLIENT by OUI..........: %02x%02x%02x (%s)\n", filterouiclient[0], filterouiclient[1], filterouiclient[2], vendor);
 	}
-if(flagfilterapless == true)		printf("filter by M2...........: requested from client (AP-LESS)\n");
-if(flagfilterrcchecked == true)		printf("filter by replaycount..: checked\n");
-if(flagfilterauthorized == true)	printf("filter by status.......: authorized (M1M4, M2M3 or M3M4)\n");
-if(flagfilternotauthorized == true)	printf("filter by status.......: challenge (M1M2)\n");
-if(pmkidwrittencount > 0)		printf("PMKID written..........: %ld\n", pmkidwrittencount);
-if(eapolwrittencount > 0)		printf("EAPOL written..........: %ld\n", eapolwrittencount);
-if(hccapxwrittencount > 0)		printf("EAPOL written to hccapx: %ld\n", hccapxwrittencount);
-if(hccapwrittencount > 0)		printf("EAPOL written to hccap.: %ld\n", hccapwrittencount);
-if(johnwrittencount > 0)		printf("EAPOL written to john..: %ld\n", johnwrittencount);
-if(essidwrittencount > 0)		printf("ESSID (unique) written.: %ld\n", essidwrittencount);
+if(flagfilterapless == true)		printf("filter by M2..................: requested from client (AP-LESS)\n");
+if(flagfilterrcchecked == true)		printf("filter by replaycount.........: checked\n");
+if(flagfilterauthorized == true)	printf("filter by status..............: authorized (M1M4, M2M3 or M3M4)\n");
+if(flagfilternotauthorized == true)	printf("filter by status..............: challenge (M1M2)\n");
+if(pmkidwrittencount > 0)		printf("PMKID written.................: %ld\n", pmkidwrittencount);
+if(eapolwrittencount > 0)		printf("EAPOL written.................: %ld\n", eapolwrittencount);
+if(hccapxwrittencount > 0)		printf("EAPOL written to hccapx.......: %ld\n", hccapxwrittencount);
+if(hccapwrittencount > 0)		printf("EAPOL written to hccap........: %ld\n", hccapwrittencount);
+if(johnwrittencount > 0)		printf("EAPOL written to john.........: %ld\n", johnwrittencount);
+if(essidwrittencount > 0)		printf("ESSID (unique) written........: %ld\n", essidwrittencount);
 printf("\n");
 return;
 }
@@ -706,13 +712,14 @@ for(zeiger = hashlist +1; zeiger < hashlist +pmkideapolcount; zeiger++)
 return;
 }
 /*===========================================================================*/
-static bool isoui(uint8_t *mac)
+static bool isoui(uint8_t *macap, uint8_t *macclient)
 {
 static ouilist_t *zeiger;
 
 for(zeiger = ouilist; zeiger < ouilist +ouicount; zeiger++)
 	{
-	if(memcmp(mac, zeiger->oui, 3) == 0) return true;
+	if(((zeiger->type &TYPE_AP) == TYPE_AP) && (memcmp(macap, zeiger->oui, 3) == 0)) return true;
+	if(((zeiger->type &TYPE_CLIENT) == TYPE_CLIENT) && (memcmp(macclient, zeiger->oui, 3) == 0)) return true;
 	}
 return false;
 }
@@ -782,9 +789,9 @@ if(filteressidpartptr != NULL)
 	{
 	if(ispartof(filteressidpartlen, (uint8_t*)filteressidpartptr, zeiger->essidlen, zeiger->essid) == false) return;
 	}
-if(filtervendorptr != 0)
+if((filtervendorptr != NULL) || (filtervendorapptr != NULL) || (filtervendorclientptr != NULL))
 	{
-	if(isoui(zeiger->ap) == false) return;
+	if(isoui(zeiger->ap, zeiger->client) == false) return;
 	}
 if((flagfilterapless == true) && ((zeiger->mp &0x10) != 0x10)) return;
 if((flagfilterrcchecked == true) && ((zeiger->mp &0x80) != 0x80)) return;
@@ -885,9 +892,9 @@ if(filteressidpartptr != NULL)
 	{
 	if(ispartof(filteressidpartlen, (uint8_t*)filteressidpartptr, zeiger->essidlen, zeiger->essid) == false) return;
 	}
-if(filtervendorptr != 0)
+if((filtervendorptr != NULL) || (filtervendorapptr != NULL) || (filtervendorclientptr != NULL))
 	{
-	if(isoui(zeiger->ap) == false) return;
+	if(isoui(zeiger->ap, zeiger->client) == false) return;
 	}
 if((flagfilterapless == true) && ((zeiger->mp &0x10) != 0x10)) return;
 if((flagfilterrcchecked == true) && ((zeiger->mp &0x80) != 0x80)) return;
@@ -1010,9 +1017,9 @@ if(filteressidpartptr != NULL)
 	{
 	if(ispartof(filteressidpartlen, (uint8_t*)filteressidpartptr, zeiger->essidlen, zeiger->essid) == false) return;
 	}
-if(filtervendorptr != 0)
+if((filtervendorptr != NULL) || (filtervendorapptr != NULL) || (filtervendorclientptr != NULL))
 	{
-	if(isoui(zeiger->ap) == false) return;
+	if(isoui(zeiger->ap, zeiger->client) == false) return;
 	}
 if((flagfilterapless == true) && ((zeiger->mp &0x10) != 0x10)) return;
 if((flagfilterrcchecked == true) && ((zeiger->mp &0x80) != 0x80)) return;
@@ -1083,7 +1090,7 @@ if((fh_essid = fopen(essidoutname, "a")) == NULL)
 	return;
 	}
 zeigerold = NULL;
-qsort(hashlist, pmkideapolcount, HASHLIST_SIZE, sort_maclist_by_essid);
+qsort(hashlist, pmkideapolcount, HASHLIST_SIZE, sort_hashlist_by_essid);
 for(pc = 0; pc < pmkideapolcount; pc++)
 	{
 	zeiger = hashlist +pc;
@@ -1122,9 +1129,9 @@ if(filteressidpartptr != NULL)
 	{
 	if(ispartof(filteressidpartlen, (uint8_t*)filteressidpartptr, zeiger->essidlen, zeiger->essid) == false) return;
 	}
-if(filtervendorptr != 0)
+if((filtervendorptr != NULL) || (filtervendorapptr != NULL) || (filtervendorclientptr != NULL))
 	{
-	if(isoui(zeiger->ap) == false) return;
+	if(isoui(zeiger->ap, zeiger->client) == false) return;
 	}
 if((flagfilterapless == true) && ((zeiger->mp &0x10) != 0x10)) return;
 if((flagfilterrcchecked == true) && ((zeiger->mp &0x80) != 0x00)) return;
@@ -1177,7 +1184,7 @@ static const char digit[16] = {'0','1','2','3','4','5','6','7','8','9','a','b','
 
 static char groupoutname[PATH_MAX];
 
-qsort(hashlist, pmkideapolcount, HASHLIST_SIZE, sort_maclist_by_essid);
+qsort(hashlist, pmkideapolcount, HASHLIST_SIZE, sort_hashlist_by_essid);
 for(zeiger = hashlist; zeiger < hashlist +pmkideapolcount; zeiger++)
 	{
 	groupoutname[0] = 0;
@@ -1219,7 +1226,7 @@ static struct stat statinfo;
 
 static char groupoutname[PATH_MAX];
 
-qsort(hashlist, pmkideapolcount, HASHLIST_SIZE, sort_maclist_by_essid);
+qsort(hashlist, pmkideapolcount, HASHLIST_SIZE, sort_hashlist_by_essid);
 for(zeiger = hashlist; zeiger < hashlist +pmkideapolcount; zeiger++)
 	{
 	snprintf(groupoutname, PATH_MAX -1, "%02x%02x%02x.22000", zeiger->ap[0], zeiger->ap[1], zeiger->ap[2]);
@@ -1249,7 +1256,7 @@ static struct stat statinfo;
 
 static char groupoutname[PATH_MAX];
 
-qsort(hashlist, pmkideapolcount, HASHLIST_SIZE, sort_maclist_by_essid);
+qsort(hashlist, pmkideapolcount, HASHLIST_SIZE, sort_hashlist_by_essid);
 for(zeiger = hashlist; zeiger < hashlist +pmkideapolcount; zeiger++)
 	{
 	snprintf(groupoutname, PATH_MAX -1, "%02x%02x%02x%02x%02x%02x.22000", zeiger->ap[0], zeiger->ap[1], zeiger->ap[2], zeiger->ap[3], zeiger->ap[4], zeiger->ap[5]);
@@ -1279,7 +1286,7 @@ static struct stat statinfo;
 
 static char groupoutname[PATH_MAX];
 
-qsort(hashlist, pmkideapolcount, HASHLIST_SIZE, sort_maclist_by_essid);
+qsort(hashlist, pmkideapolcount, HASHLIST_SIZE, sort_hashlist_by_essid);
 for(zeiger = hashlist; zeiger < hashlist +pmkideapolcount; zeiger++)
 	{
 	snprintf(groupoutname, PATH_MAX -1, "%02x%02x%02x%02x%02x%02x.22000", zeiger->client[0], zeiger->client[1], zeiger->client[2], zeiger->client[3], zeiger->client[4], zeiger->client[5]);
@@ -1322,7 +1329,7 @@ if(pmkideapoloutname != NULL)
 		return;
 		}
 	}
-qsort(hashlist, pmkideapolcount, HASHLIST_SIZE, sort_maclist_by_essid);
+qsort(hashlist, pmkideapolcount, HASHLIST_SIZE, sort_hashlist_by_essid);
 
 zeigerbegin = hashlist;
 lc = 0;
@@ -1403,9 +1410,9 @@ if(filteressidpartptr != NULL)
 	{
 	if(ispartof(filteressidpartlen, (uint8_t*)filteressidpartptr, zeiger->essidlen, zeiger->essid) == false) return;
 	}
-if(filtervendorptr != 0)
+if((filtervendorptr != NULL) || (filtervendorapptr != NULL) || (filtervendorclientptr != NULL))
 	{
-	if(isoui(zeiger->ap) == false) return;
+	if(isoui(zeiger->ap, zeiger->client) == false) return;
 	}
 if((flagfilterapless == true) && ((zeiger->mp &0x10) != 0x10)) return;
 if((flagfilterrcchecked == true) && ((zeiger->mp &0x80) != 0x00)) return;
@@ -1460,6 +1467,90 @@ fprintf(fh_pmkideapol, "\n");
 return;
 }
 /*===========================================================================*/
+static void writevendorapinfofile(char *vendorinfooutname)
+{
+static char *vendor;
+static hashlist_t *zeiger;
+static FILE *fh_info;
+
+qsort(hashlist, pmkideapolcount, HASHLIST_SIZE, sort_hashlist_by_macap);
+if(strcmp(vendorinfooutname, "stdout") != 0)
+	{
+	if((fh_info = fopen(vendorinfooutname, "a")) == NULL)
+		{
+		printf("error opening file %s: %s\n", vendorinfooutname, strerror(errno));
+		return;
+		}
+	vendor = getvendor(hashlist->ap);
+	fprintf(stdout, "%02x%02x%02x\t%s\t[ACCESS POINT]\n", hashlist->ap[0], hashlist->ap[1], hashlist->ap[2], vendor);
+	for(zeiger = hashlist +1; zeiger < hashlist +pmkideapolcount; zeiger++)
+		{
+		if(memcmp((zeiger -1)->ap, zeiger->ap, 3) != 0)
+			{
+			vendor = getvendor(zeiger->ap);
+			fprintf(fh_info, "%02x%02x%02x\t%s\t[ACCESS POINT]\n", zeiger->ap[0], zeiger->ap[1], zeiger->ap[2], vendor);
+			}
+		}
+	fclose(fh_info);
+	}
+else
+	{
+	vendor = getvendor(hashlist->ap);
+	fprintf(stdout, "%02x%02x%02x\t%s\t[ACCESS POINT]\n", hashlist->ap[0], hashlist->ap[1], hashlist->ap[2], vendor);
+	for(zeiger = hashlist +1; zeiger < hashlist +pmkideapolcount; zeiger++)
+		{
+		if(memcmp((zeiger -1)->ap, zeiger->ap, 3) != 0)
+			{
+			vendor = getvendor(zeiger->ap);
+			fprintf(stdout, "%02x%02x%02x\t%s\t[ACCESS POINT]\n", zeiger->ap[0], zeiger->ap[1], zeiger->ap[2], vendor);
+			}
+		}
+	}
+return;
+}
+/*===========================================================================*/
+static void writevendorclientinfofile(char *vendorinfooutname)
+{
+static char *vendor;
+static hashlist_t *zeiger;
+static FILE *fh_info;
+
+qsort(hashlist, pmkideapolcount, HASHLIST_SIZE, sort_hashlist_by_macclient);
+if(strcmp(vendorinfooutname, "stdout") != 0)
+	{
+	if((fh_info = fopen(vendorinfooutname, "a")) == NULL)
+		{
+		printf("error opening file %s: %s\n", vendorinfooutname, strerror(errno));
+		return;
+		}
+	vendor = getvendor(hashlist->client);
+	fprintf(stdout, "%02x%02x%02x\t%s\t[CLIENT]\n", hashlist->client[0], hashlist->client[1], hashlist->client[2], vendor);
+	for(zeiger = hashlist +1; zeiger < hashlist +pmkideapolcount; zeiger++)
+		{
+		if(memcmp((zeiger -1)->client, zeiger->client, 3) != 0)
+			{
+			vendor = getvendor(zeiger->client);
+			fprintf(fh_info, "%02x%02x%02x\t%s\t[CLIENT]\n", zeiger->client[0], zeiger->client[1], zeiger->client[2], vendor);
+			}
+		}
+	fclose(fh_info);
+	}
+else
+	{
+	vendor = getvendor(hashlist->ap);
+	fprintf(stdout, "%02x%02x%02x\t%s\t[CLIENT]\n", hashlist->ap[0], hashlist->ap[1], hashlist->ap[2], vendor);
+	for(zeiger = hashlist +1; zeiger < hashlist +pmkideapolcount; zeiger++)
+		{
+		if(memcmp((zeiger -1)->ap, zeiger->ap, 3) != 0)
+			{
+			vendor = getvendor(zeiger->ap);
+			fprintf(stdout, "%02x%02x%02x\t%s\t[CLIENT]\n", zeiger->ap[0], zeiger->ap[1], zeiger->ap[2], vendor);
+			}
+		}
+	}
+return;
+}
+/*===========================================================================*/
 static void writeinfofile(char *infooutname)
 {
 static hashlist_t *zeiger;
@@ -1511,7 +1602,6 @@ while((lineptr[p] != '*') && (lineptr[p] != 0) && (p /2 <= bufflen))
 	idx1 = ((uint8_t)lineptr[p +1] &0x1F) ^0x10;
 	buff[p /2] = (uint8_t)(hashmap[idx0] <<4) | hashmap[idx1];
 	p += 2;
-	if((p /2) > PMKIDEAPOL_BUFFER_LEN) return 0;
 	}
 return p /2;
 }
@@ -1549,6 +1639,110 @@ len = chop(buffptr, len);
 return len;
 }
 /*===========================================================================*/
+static void removepmkideapol(char *macskipname)
+{
+static int len;
+static int p1, p2;
+static FILE *fh_maclistin;
+static long int i, f, r;
+
+static int maclistskipcount, maclistskipmax;
+static maclist_t *maclistskip, *zeiger, *maclistskipnew;
+static hashlist_t *zeigerhash;
+
+static char linein[PMKIDEAPOL_BUFFER_LEN];
+
+maclistskipmax = 1000;
+if((maclistskip = (maclist_t*)calloc(maclistskipmax, MACLIST_SIZE)) == NULL) return;
+if((fh_maclistin = fopen(macskipname, "rb")) == NULL)
+	{
+	printf("error opening file %s: %s\n", macskipname, strerror(errno));
+	return;
+	}
+
+zeiger = maclistskip;
+maclistskipcount = 0;
+while(1)
+	{
+	if((len = fgetline(fh_maclistin, PMKIDEAPOL_BUFFER_LEN, linein)) == -1) break;
+	if(len == 17)
+		{
+		p2 = 0;
+		for(p1 = 0; p1 < 17; p1++)
+			{
+			if(isxdigit(linein[p1]))
+				{
+				linein[p2] = linein[p1];
+				p2++;
+				}
+			}
+		linein[p2] = 0;
+		len = p2;
+		}
+	if(len != 12) continue;
+	if(getfield(linein, 6, zeiger->mac) != 6) continue;
+	maclistskipcount++;
+	if(maclistskipcount >= maclistskipmax)
+		{
+		maclistskipmax += 1000;
+		maclistskipnew = realloc(maclistskip, maclistskipmax *MACLIST_SIZE);
+		if(maclistskipnew == NULL)
+			{
+			printf("failed to allocate memory for internal list\n");
+			exit(EXIT_FAILURE);
+			}
+		maclistskip = maclistskipnew;
+		}
+	zeiger = maclistskip +maclistskipcount;
+	}
+if(fh_maclistin != NULL) fclose(fh_maclistin);
+qsort(maclistskip, maclistskipcount, MACLIST_SIZE, sort_maclistin);
+qsort(hashlist, pmkideapolcount, HASHLIST_SIZE, sort_hashlist_by_macap);
+zeigerhash = hashlist;
+zeiger = maclistskip;
+f = 0;
+r = 0;
+for(i = 0; i < pmkideapolcount; i++)
+	{
+	if(memcmp((zeigerhash +i)->ap, (zeiger +f)->mac, 6) > 0)
+	while(f < maclistskipcount)
+		{
+		if(memcmp((zeiger +f)->mac, (zeigerhash +i)->ap, 6) >= 0) break;
+		f++;
+		}
+	if(memcmp((zeigerhash +i)->ap, (zeiger +f)->mac, 6) == 0)
+		{
+		(zeigerhash +i)->type = HS_REMOVED;
+		r++;
+		}
+	}
+qsort(hashlist, pmkideapolcount, HASHLIST_SIZE, sort_hashlist_by_type);
+pmkidcount -= r;
+qsort(hashlist, pmkideapolcount, HASHLIST_SIZE, sort_hashlist_by_macclient);
+zeigerhash = hashlist;
+zeiger = maclistskip;
+f = 0;
+r = 0;
+for(i = 0; i < pmkideapolcount; i++)
+	{
+	if(memcmp((zeigerhash +i)->client, (zeiger +f)->mac, 6) > 0)
+	while(f < maclistskipcount)
+		{
+		if(memcmp((zeiger +f)->mac, (zeigerhash +i)->client, 6) >= 0) break;
+		f++;
+		}
+	if(memcmp((zeigerhash +i)->client, (zeiger +f)->mac, 6) == 0)
+		{
+		(zeigerhash +i)->type = HS_REMOVED;
+		r++;
+		}
+	}
+qsort(hashlist, pmkideapolcount, HASHLIST_SIZE, sort_hashlist_by_type);
+pmkidcount -= r;
+if(maclistskip != NULL) free(maclistskip);
+return;
+}
+/*===========================================================================*/
 static void processmacfile(char *maclistinname, char *pmkideapoloutname)
 {
 static int len;
@@ -1571,7 +1765,6 @@ if((fh_maclistin = fopen(maclistinname, "rb")) == NULL)
 	printf("error opening file %s: %s\n", maclistinname, strerror(errno));
 	return;
 	}
-
 zeiger = maclistin;
 maclistincount = 0;
 while(1)
@@ -1619,7 +1812,7 @@ if(pmkideapoloutname != NULL)
 		return;
 		}
 	}
-qsort(hashlist, pmkideapolcount, HASHLIST_SIZE, sort_maclist_by_macap);
+qsort(hashlist, pmkideapolcount, HASHLIST_SIZE, sort_hashlist_by_macap);
 zeiger = maclistin;
 zeigerhash = hashlist;
 o = 0;
@@ -1632,7 +1825,7 @@ for(i = 0; i < maclistincount; i++)
 		o++;
 		}
 	}
-qsort(hashlist, pmkideapolcount, HASHLIST_SIZE, sort_maclist_by_macclient);
+qsort(hashlist, pmkideapolcount, HASHLIST_SIZE, sort_hashlist_by_macclient);
 zeiger = maclistin;
 zeigerhash = hashlist;
 o = 0;
@@ -1713,7 +1906,7 @@ while(1)
 	}
 if(fh_essidlistin != NULL) fclose(fh_essidlistin);
 qsort(essidlistin, essidlistincount, ESSIDLIST_SIZE, sort_essidlistin);
-qsort(hashlist, pmkideapolcount, HASHLIST_SIZE, sort_maclist_by_essidlen);
+qsort(hashlist, pmkideapolcount, HASHLIST_SIZE, sort_hashlist_by_essidlen);
 if(pmkideapoloutname == NULL)
 	{
 	if(essidlistin != NULL) free(essidlistin);
@@ -1874,6 +2067,51 @@ while(1)
 		}
 	zeiger = hashlist +pmkideapolcount;
 	}
+return true;
+}
+/*===========================================================================*/
+static bool readbpkdf2file(char *pkdf2inname)
+{
+static int len;
+static char *pskpos;
+static FILE *fh_pbkdf2;
+static char linein[PBKDF2_LINE_LEN +1];
+
+if((fh_pbkdf2 = fopen(pkdf2inname, "r")) == NULL)
+	{
+	printf("error opening file %s: %s\n", pkdf2inname, strerror(errno));
+	return false;
+	}
+
+pbkdf2count = 0;
+pbkdf2readerrorcount = 0;
+while(1)
+	{
+	if((len = fgetline(fh_pbkdf2, PBKDF2_LINE_LEN, linein)) == -1) break;
+	if(len < 76)
+		{
+		pbkdf2readerrorcount++;
+		continue;
+		}
+	if(linein[64] != '*')
+		{
+		pbkdf2readerrorcount++;
+		continue;
+		}
+	pskpos = strchr(&linein[65], ':');
+	if(pskpos == NULL)
+		{
+		pbkdf2readerrorcount++;
+		continue;
+		}
+	pskpos[0] = 0;
+
+	printf("%s %s\n", &linein[65], pskpos +1);
+	pbkdf2count++;
+	}
+fclose(fh_pbkdf2);
+
+
 return true;
 }
 /*===========================================================================*/
@@ -1885,6 +2123,32 @@ for(zeiger = ouilist; zeiger < ouilist +ouicount; zeiger++) fprintf(stdout, "%02
 return;
 }
 /*===========================================================================*/
+static int isvendor(int len, char *linein)
+{
+static int c;
+static int ret;
+
+for(c = 7; c < len; c++)
+	{
+	if(islower(linein[c])) linein[c] = toupper(linein[c]);
+	}
+
+ret = 0;
+if(filtervendorptr != NULL)
+	{
+	if(strstr(&linein[7], filtervendorptr) != NULL) ret |= TYPE_AP + TYPE_CLIENT;
+	}
+if(filtervendorapptr != NULL)
+	{
+	if(strstr(&linein[7], filtervendorapptr) != NULL) ret |= TYPE_AP;
+	}
+if(filtervendorclientptr != NULL)
+	{
+	if(strstr(&linein[7], filtervendorclientptr) != NULL) ret |= TYPE_CLIENT;
+	}
+return ret;
+}
+/*===========================================================================*/
 static void readoui()
 {
 static int len;
@@ -1919,9 +2183,11 @@ while(1)
 	linein[6] = 0;
 	if(getfield(linein, OUI_LINE_LEN, zeiger->oui) != 3) continue;
 	if(strstr(&linein[7], "(base 16)") == NULL) continue;
-	if(filtervendorptr != NULL)
+	zeiger->type = 0;
+	if((filtervendorptr != NULL) || (filtervendorapptr != NULL) || (filtervendorclientptr != NULL))
 		{
-		if(strstr(&linein[7], filtervendorptr) == NULL) continue;
+		zeiger->type = isvendor(len, linein);
+		if(zeiger->type == 0) continue;
 		}
 	vendorptr = strrchr(&linein[7], '\t');
 	if(vendorptr == NULL) continue;
@@ -1980,9 +2246,9 @@ if((fhoui = fopen(ouinameuserpath, "w")) == NULL)
 	}
 hnd = curl_easy_init ();
 curl_easy_setopt(hnd, CURLOPT_URL, "http://standards-oui.ieee.org/oui/oui.txt");
-curl_easy_setopt(hnd, CURLOPT_NOPROGRESS, 1L);
 curl_easy_setopt(hnd, CURLOPT_MAXREDIRS, 5L);
 curl_easy_setopt(hnd, CURLOPT_WRITEDATA, fhoui) ;
+curl_easy_setopt(hnd, CURLOPT_NOPROGRESS, 0L);
 ret = curl_easy_perform(hnd);
 curl_easy_cleanup(hnd);
 fclose(fhoui);
@@ -2016,6 +2282,7 @@ printf("%s %s (C) %s ZeroBeat\n"
 	"-d          : download http://standards-oui.ieee.org/oui.txt\n"
 	"              and save to ~/.hcxtools/oui.txt\n"
 	"              internet connection required\n"
+//	"-p          : input PBKDF2 file (hashcat potfile 22000 format)\n" 
 	"-h          : show this help\n"
 	"-v          : show version\n"
 	"\n"
@@ -2034,7 +2301,7 @@ printf("%s %s (C) %s ZeroBeat\n"
 	"                                2 = EAPOL\n"
 	"                               default PMKID and EAPOL (1+2=3)\n"
 	"--hcx-min=<digit>            : disregard hashes with occurrence lower than hcx-min/ESSID\n"
-	"--hcx-max=<digit>            : disregard hashes with occurrence higher than hcx-min/ESSID\n"
+	"--hcx-max=<digit>            : disregard hashes with occurrence higher than hcx-max/ESSID\n"
 	"--essid-len                  : filter by ESSID length\n"
 	"                               default ESSID length: %d...%d\n"
 	"--essid-min                  : filter by ESSID minimum length\n"
@@ -2050,18 +2317,35 @@ printf("%s %s (C) %s ZeroBeat\n"
 	"                               format: 001122334455, 00:11:22:33:44:55, 00-11-22-33-44-55 (hex)\n"
 	"--mac-list=<file>            : filter by MAC file\n"
 	"                               format: 001122334455, 00:11:22:33:44:55, 00-11-22-33-44-55 (hex)\n"
+	"--mac-skiplist=<file>        : exclude MAC from file\n"
+	"                               format: 001122334455, 00:11:22:33:44:55, 00-11-22-33-44-55 (hex)\n"
 	"--oui-ap=<OUI>               : filter AP by OUI\n"
 	"                               format: 001122, 00:11:22, 00-11-22 (hex)\n"
 	"--oui-client=<OUI>           : filter CLIENT by OUI\n"
 	"                               format: 001122, 00:11:22, 00-11-22 (hex)\n"
-	"--vendor=<VENDOR>            : filter by (part of) VENDOR name\n"
+	"--vendor=<VENDOR>            : filter AP or CLIENT by (part of) VENDOR name\n"
+	"--vendor-ap=<VENDOR>         : filter AP by (part of) VENDOR name\n"
+	"--vendor-client=<VENDOR>     : filter CLIENT by (part of) VENDOR name\n"
 	"--authorized                 : filter EAPOL pairs by status authorized\n"
 	"--notauthorized              : filter EAPOL pairs by status CHALLENGE (not authorized)\n"
 	"--rc                         : filter EAPOL pairs by replaycount status checked\n"
 	"--apless                     : filter EAPOL pairs by status M1M2ROGUE (M2 requested from CLIENT)\n"
 	"--info=<file>                : output detailed information about content of hash file\n"
+	"                               not in combination with --vendor, --vendor-ap or --vendor-client\n"
 	"--info=stdout                : stdout output detailed information about content of hash file\n"
-	"--vendorlist                 : stdout output VENDOR list sorted by OUI\n"
+	"                               not in combination with --vendor, --vendor-ap or --vendor-client\n"
+	"--info-vendor=<file>         : output detailed information about ACCESS POINT and CLIENT VENDORs\n"
+	"                               not in combination with --vendor, --vendor-ap or --vendor-client\n"
+	"--info-vendor-ap=<file>      : output detailed information about ACCESS POINT VENDORs\n"
+	"                               not in combination with --vendor, --vendor-ap or --vendor-client\n"
+	"--info-vendor-client=<file>  : output detailed information about ACCESS POINT VENDORs\n"
+	"                               not in combination with --vendor, --vendor-ap or --vendor-client\n"
+	"--info-vendor=stdout         : stdout output detailed information about ACCESS POINT and CLIENT VENDORs\n"
+	"                               not in combination with --vendor, --vendor-ap or --vendor-client\n"
+	"--info-vendor-ap=stdout      : stdout output detailed information about ACCESS POINT VENDORs\n"
+	"                               not in combination with --vendor, --vendor-ap or --vendor-client\n"
+	"--info-vendor-client=stdout  : stdout output detailed information about ACCESS POINT VENDORs\n"
+	"                               not in combination with --vendor, --vendor-ap or --vendor-client\n"
 	"--psk=<PSK>                  : pre-shared key to test\n"
 	"                               due to PBKDF2 calculation this is a very slow process\n"
 	"                               no nonce error corrections\n"
@@ -2071,6 +2355,7 @@ printf("%s %s (C) %s ZeroBeat\n"
 	"--hccap=<file>               : output to ancient hccap file\n"
 	"--hccap-single               : output to ancient hccap single files (MAC + count)\n"
 	"--john=<file>                : output to deprecated john file\n"
+	"--vendorlist                 : stdout output complete OUI list sorted by OUI\n"
 	"--help                       : show this help\n"
 	"--version                    : show version\n"
 	"\n", eigenname, VERSION_TAG, VERSION_YEAR, eigenname, ESSID_LEN_MIN, ESSID_LEN_MAX, ESSID_LEN_MIN, ESSID_LEN_MAX);
@@ -2096,20 +2381,25 @@ static int p1, p2;
 static int hashtypein;
 static int essidlenin;
 static FILE *fh_pmkideapol;
+static char *pbkdf2inname;
 static char *pmkideapolinname;
 static char *pmkideapoloutname;
 static char *essidoutname;
 static char *essidinname;
 static char *macinname;
+static char *macskipname;
 static char *hccapxoutname;
 static char *hccapoutname;
 static char *johnoutname;
 static char *infooutname;
+static char *infovendoroutname;
+static char *infovendorapoutname;
+static char *infovendorclientoutname;
 static char *ouiinstring;
 static char *macinstring;
 static char *pmkinstring;
 
-static const char *short_options = "i:o:E:dhv";
+static const char *short_options = "i:o:E:dp:hv";
 static const struct option long_options[] =
 {
 	{"type",			required_argument,	NULL,	HCX_HASH_TYPE},
@@ -2126,24 +2416,30 @@ static const struct option long_options[] =
 	{"mac-ap",			required_argument,	NULL,	HCX_FILTER_MAC_AP},
 	{"mac-client",			required_argument,	NULL,	HCX_FILTER_MAC_CLIENT},
 	{"mac-list",			required_argument,	NULL,	HCX_FILTER_MAC_LIST_IN},
+	{"mac-skiplist",		required_argument,	NULL,	HCX_FILTER_MAC_LIST_SKIP},
 	{"mac-group-ap",		no_argument,		NULL,	HCX_MAC_GROUP_AP},
 	{"mac-group-client",		no_argument,		NULL,	HCX_MAC_GROUP_CLIENT},
 	{"oui-group",			no_argument,		NULL,	HCX_OUI_GROUP},
 	{"oui-ap",			required_argument,	NULL,	HCX_FILTER_OUI_AP},
-	{"vendor",			required_argument,	NULL,	HCX_FILTER_VENDOR},
 	{"oui-client",			required_argument,	NULL,	HCX_FILTER_OUI_CLIENT},
+	{"vendor",			required_argument,	NULL,	HCX_FILTER_VENDOR},
+	{"vendor-ap",			required_argument,	NULL,	HCX_FILTER_VENDOR_AP},
+	{"vendor-client",		required_argument,	NULL,	HCX_FILTER_VENDOR_CLIENT},
 	{"rc",				no_argument,		NULL,	HCX_FILTER_RC},
 	{"authorized",			no_argument,		NULL,	HCX_FILTER_M12},
 	{"notauthorized",		no_argument,		NULL,	HCX_FILTER_M1234},
 	{"apless",			no_argument,		NULL,	HCX_FILTER_M1M2ROGUE},
 	{"psk",				required_argument,	NULL,	HCX_PSK},
 	{"pmk",				required_argument,	NULL,	HCX_PMK},
-	{"vendorlist",			no_argument,		NULL,	HCX_VENDOR_OUT},
 	{"info",			required_argument,	NULL,	HCX_INFO_OUT},
+	{"info-vendor",			required_argument,	NULL,	HCX_INFO_VENDOR_OUT},
+	{"info-vendor-ap",		required_argument,	NULL,	HCX_INFO_VENDOR_AP_OUT},
+	{"info-vendor-client",		required_argument,	NULL,	HCX_INFO_VENDOR_CLIENT_OUT},
 	{"hccapx",			required_argument,	NULL,	HCX_HCCAPX_OUT},
 	{"hccap",			required_argument,	NULL,	HCX_HCCAP_OUT},
 	{"hccap-single",		no_argument,		NULL,	HCX_HCCAP_SINGLE_OUT},
 	{"john",			required_argument,	NULL,	HCX_JOHN_OUT},
+	{"vendorlist",			no_argument,		NULL,	HCX_VENDOR_OUT},
 	{"version",			no_argument,		NULL,	HCX_VERSION},
 	{"help",			no_argument,		NULL,	HCX_HELP},
 	{NULL,				0,			NULL,	0}
@@ -2153,13 +2449,18 @@ auswahl = -1;
 index = 0;
 optind = 1;
 optopt = 0;
+pbkdf2inname = NULL;
 fh_pmkideapol = NULL;
 pmkideapolinname = NULL;
 pmkideapoloutname = NULL;
 essidoutname = NULL;
 essidinname = NULL;
 macinname = NULL;
+macskipname = NULL;
 infooutname = NULL;
+infovendoroutname = NULL;
+infovendorapoutname = NULL;
+infovendorclientoutname = NULL;
 hccapxoutname = NULL;
 hccapoutname = NULL;
 johnoutname = NULL;
@@ -2169,6 +2470,8 @@ pmkinstring = NULL;
 filteressidptr = NULL;
 filteressidpartptr = NULL;
 filtervendorptr = NULL;
+filtervendorapptr = NULL;
+filtervendorclientptr = NULL;
 flagfiltermacap = false;
 flagfiltermacclient = false;
 flagfilterouiap = false;
@@ -2218,15 +2521,27 @@ while((auswahl = getopt_long (argc, argv, short_options, long_options, &index))
 		infooutname = optarg;
 		break;
 
+		case HCX_INFO_VENDOR_OUT:
+		infovendoroutname = optarg;
+		break;
+
+		case HCX_INFO_VENDOR_AP_OUT:
+		infovendorapoutname = optarg;
+		break;
+
+		case HCX_INFO_VENDOR_CLIENT_OUT:
+		infovendorclientoutname = optarg;
+		break;
+
 		case HCX_ESSID_GROUP:
 		flagessidgroup = true;
 		break;
 
 		case HCX_HASH_TYPE:
 		hashtypein |= strtol(optarg, NULL, 10);
-		if((hashtypein < HCX_TYPE_PMKID) || (hashtypein < HCX_TYPE_EAPOL))
+		if((hashtypein < HCX_TYPE_PMKID) || (hashtypein > (HCX_TYPE_PMKID + HCX_TYPE_EAPOL)))
 			{
-			fprintf(stderr, "only hash types 1 and 2 allowed\n");
+			fprintf(stderr, "only hash types 1 and 2 allowed (values 1, 2 or 3)\n");
 			exit(EXIT_FAILURE);
 			}
 		break;
@@ -2373,6 +2688,10 @@ while((auswahl = getopt_long (argc, argv, short_options, long_options, &index))
 		macinname = optarg;
 		break;
 
+		case HCX_FILTER_MAC_LIST_SKIP:
+		macskipname = optarg;
+		break;
+
 		case HCX_FILTER_OUI_CLIENT:
 		l= strlen(optarg);
 		p2 = 0;
@@ -2396,6 +2715,44 @@ while((auswahl = getopt_long (argc, argv, short_options, long_options, &index))
 
 		case HCX_FILTER_VENDOR:
 		filtervendorptr = optarg;
+		l = strlen(filtervendorptr);
+		if(l < 3)
+			{
+			fprintf(stderr, "at least three characters of the VENDOR name are mandatory\n");
+			exit(EXIT_FAILURE);
+			}
+		for(p1 = 0; p1 < l; p1++)
+			{
+			if(islower(filtervendorptr[p1])) filtervendorptr[p1] = toupper(filtervendorptr[p1]);
+			}
+		break;
+
+		case HCX_FILTER_VENDOR_AP:
+		filtervendorapptr = optarg;
+		l = strlen(filtervendorapptr);
+		if(l < 3)
+			{
+			fprintf(stderr, "at least three characters of the VENDOR name are mandatory\n");
+			exit(EXIT_FAILURE);
+			}
+		for(p1 = 0; p1 < l; p1++)
+			{
+			if(islower(filtervendorapptr[p1])) filtervendorapptr[p1] = toupper(filtervendorapptr[p1]);
+			}
+		break;
+
+		case HCX_FILTER_VENDOR_CLIENT:
+		filtervendorclientptr = optarg;
+		l = strlen(filtervendorclientptr);
+		if(l < 3)
+			{
+			fprintf(stderr, "at least three characters of the VENDOR name are mandatory\n");
+			exit(EXIT_FAILURE);
+			}
+		for(p1 = 0; p1 < l; p1++)
+			{
+			if(islower(filtervendorclientptr[p1])) filtervendorclientptr[p1] = toupper(filtervendorclientptr[p1]);
+			}
 		break;
 
 		case HCX_FILTER_RC:
@@ -2439,6 +2796,10 @@ while((auswahl = getopt_long (argc, argv, short_options, long_options, &index))
 		downloadoui();
 		break;
 
+		case HCX_PBKDF2_IN:
+//		pbkdf2inname = optarg;
+		break;
+
 		case HCX_HCCAPX_OUT:
 		hccapxoutname = optarg;
 		break;
@@ -2483,6 +2844,14 @@ if(argc < 2)
 
 if(initlists() == false) exit(EXIT_FAILURE);
 
+if(pbkdf2inname != NULL) readbpkdf2file(pbkdf2inname);
+
+if((infooutname != NULL) || (infovendoroutname != NULL) || (infovendorapoutname != NULL) || (infovendorclientoutname != NULL))
+	{
+	filtervendorptr = NULL;
+	filtervendorapptr = NULL;
+	filtervendorclientptr = NULL;
+	}
 readoui();
 if((ouicount > 0) && (flagvendorout == true))
 	{
@@ -2503,6 +2872,8 @@ if(pmkideapolinname != NULL)
 	}
 
 if(fh_pmkideapol != NULL) readpmkideapolfile(fh_pmkideapol);
+if((pmkideapolcount > 0) && (macskipname != NULL)) removepmkideapol(macskipname);
+
 if(hashtypein > 0) hashtype = hashtypein;
 
 if((pmkideapolcount > 0) && (essidoutname != NULL)) processessid(essidoutname);
@@ -2511,7 +2882,6 @@ if((pmkideapolcount > 0) && (pmkideapoloutname != NULL) && (essidinname == NULL)
 	if((lcmin == 0) && (lcmax == 0)) writeeapolpmkidfile(pmkideapoloutname);
 	else writelceapolpmkidfile(pmkideapoloutname, lcmin, lcmax);
 	}
-if((pmkideapolcount > 0) && (infooutname != NULL)) writeinfofile(infooutname);
 if((pmkideapolcount > 0) && (flagessidgroup == true)) writeeapolpmkidessidgroups();
 if((pmkideapolcount > 0) && (flagmacapgroup == true)) writeeapolpmkidmacapgroups();
 if((pmkideapolcount > 0) && (flagmacclientgroup == true)) writeeapolpmkidmacclientgroups();
@@ -2524,8 +2894,16 @@ if((pmkideapolcount > 0) && (flaghccapsingleout == true)) writehccapsinglefile()
 if((pmkideapolcount > 0) && (johnoutname != NULL)) writejohnfile(johnoutname);
 if((pmkideapolcount > 0) && (pmkideapoloutname != NULL) && (essidinname != NULL)) processessidfile(essidinname, pmkideapoloutname);
 if((pmkideapolcount > 0) && (macinname != NULL)) processmacfile(macinname, pmkideapoloutname);
+if((pmkideapolcount > 0) && (infooutname != NULL)) writeinfofile(infooutname);
+if((pmkideapolcount > 0) && (infovendoroutname != NULL))
+	{
+	writevendorapinfofile(infovendoroutname);
+	writevendorclientinfofile(infovendoroutname);
+	}
+else if((pmkideapolcount > 0) && (infovendorapoutname != NULL)) writevendorapinfofile(infovendorapoutname);
+else if((pmkideapolcount > 0) && (infovendorclientoutname != NULL)) writevendorclientinfofile(infovendorclientoutname);
 
-printstatus();
+if((infooutname == NULL) && (infovendoroutname == NULL) && (infovendorapoutname == NULL) && (infovendorclientoutname == NULL)) printstatus();
 if(fh_pmkideapol != NULL) fclose(fh_pmkideapol);
 closelists();
 return EXIT_SUCCESS;
