@@ -29,7 +29,7 @@
 
 static apessidl_t *apessidliste;
 static int apessidcount;
-static int thisyear;
+static unsigned int thisyear;
 
 static bool airtelflag;
 static bool askeyarrisflag;
@@ -40,6 +40,8 @@ static bool egnflag;
 static bool eudateflag;
 static bool hb5flag;
 static bool netgearflag;
+static bool maconlyflag;
+static bool noessidcombinationflag;
 static bool phomeflag;
 static bool podaflag;
 static bool tendaflag;
@@ -77,6 +79,7 @@ static char upperpskstring[PSKSTRING_LEN_MAX] = {};
 
 l = strlen(pskstring);
 if((l < 8) || (l > PSKSTRING_LEN_MAX)) return;
+fprintf(fhout,"%s\n", pskstring);
 for(p = 0; p < l; p++)
 	{
 	if(islower(pskstring[p]))
@@ -92,7 +95,8 @@ for(p = 0; p < l; p++)
 		}
 	else lowerpskstring[p] = pskstring[p];
 	}
-fprintf(fhout,"%s\n", pskstring);
+upperpskstring[p] = 0;
+lowerpskstring[p] = 0;
 if(uflag == true) fprintf(fhout,"%s\n", upperpskstring);
 if(lflag == true) fprintf(fhout,"%s\n", lowerpskstring);
 if((lowerpskstring[0] >= 'a') && (lowerpskstring[0] <= 'z'))
@@ -171,9 +175,10 @@ static const char *firstword[] =
 { 
 "ancient", "aquatic",
 "basic", "black", "blue", "bold", "brave", "breezy", "bright",
-"calm", "cheerful", "chummy", "classy", "clever", "cloudy", "cool", "crispy", "curly",
+"calm", "cheerful", "chummy", "classy", "clever", "cloudy", "cool", "crispy",
+"curly",
 "daily", "deep", "delightful", "dizzy", "dynamic",
-"elated", "elegant", "excited", "exotic",
+"eagle", "elated", "elegant", "excited", "exotic",
 "fancy", "fearless", "festive", "fluffy", "free", "fresh", "friendly", "fuzzy",
 "gentle", "gifted", "gigantic", "graceful", "grand", "great", "green",
 "happy", "heavy", "helpful", "hungry", "husky",
@@ -187,8 +192,9 @@ static const char *firstword[] =
 "pastel", "perfect", "phobic", "pink", "polite", "precious", "purple",
 "quaint", "quick", "quiet",
 "rapid", "red", "rocky", "round", "royal", "rustic",
-"shiny", "silent", "silky", "silly", "slow", "smiling", "smooth", "strong", "sunny", "sweet",
-"thirsty", "thoughtful", "tiny",
+"shiny", "silent", "silky", "silly", "slow", "smiling", "smooth", "strong",
+"sunny", "sweet",
+"tablet", "thirsty", "thoughtful", "tiny",
 "uneven", "unusual",
 "vanilla", "vast",
 "watery", "wide", "windy", "witty", "wonderful",
@@ -199,9 +205,11 @@ static const char *firstword[] =
 static const char *secondword[] =
 {
 "airplane", "apple",
-"balloon", "banana", "berry", "bird", "boat", "bolt", "box", "brain", "breeze", "bug", "butter",
-"canoe", "car", "carrot", "cartoon", "cello", "chair", "cheese", "coconut", "comet", "cream", "curtain",
-"daisy", "diamond",
+"balloon", "banana", "bay", "berry", "bird", "boat", "bolt", "box",
+"brain", "breeze", "bug", "butter",
+"canoe", "car", "carrot", "cartoon", "cello", "chair", "cheese", "coconut",
+"comet", "cream", "curtain",
+"daisy", "diamond", "domain",
 "earth", "ecasa", "elephant",
 "finch", "fire", "fish", "flamingo", "flower", "flute",
 "gadfly", "giant", "grasshopper",
@@ -209,16 +217,19 @@ static const char *secondword[] =
 "ink", "iris",
 "jade", "jet", "jetcar", "jungle",
 "kangaroo", "kayak",
-"lake", "lightning", "lotus",
-"mango", "mesa", "mint", "moon", "mountain",
+"lake", "lightning", "link", "lotus",
+"mango", "mesa", "mint", "mobile", "moon", "mountain",
 "nest",
 "oboe", "ocean", "octopus", "onion", "orchestra", "owl",
-"panda", "phoenix", "piano", "pineapple", "planet", "plum", "pond", "poodle", "potato", "prairie",
+"panda", "phoenix", "piano", "pineapple", "planet", "plum", "pond", "poodle",
+"potato", "prairie",
 "quail",
 "rabbit", "raccoon", "raven", "river", "road", "rosebud",
-"sea", "sheep", "ship", "shoe", "shrub", "skates", "sky", "snail", "socks", "sparrow", "spider", "squash",
+"sea", "sheep", "ship", "shoe", "shrub", "skates", "sky", "snail",
+"socks", "sparrow", "spider", "stone", "squash",
 "squirrel", "star", "street", "sun",
-"table", "teapot", "tomato", "trail", "train", "tree", "trumpet", "tuba", "tulip", "turkey", 
+"table", "tail", "teapot", "time", "tomato", "trail", "train", "tree",
+"trumpet", "tuba", "tulip", "turkey", 
 "umbrella", "unicorn", "unit",
 "valley", "vase", "violet", "violin",
 "water", "wind", "window",
@@ -282,23 +293,24 @@ static char pskstring[PSKSTRING_LEN_MAX] = {};
 
 static const char *wordlist[] =
 { 
-"absent", "absurd", "acre", "adorable", "advice", "agency", "ago", "airplane",
-"album", "anchor", "ancient", "apple", "aquatic", "author", "away",
+"absent", "absurd", "acre", "ad", "adorable", "advice", "agency", "ago",
+"airplane", "album", "anchor", "ancient", "angry", "apple", "aquatic", "author",
+"away",
 "bakery", "ball", "balloon", "banana", "barrel", "basic", "basket", "bench",
 "berry", "better", "bike", "bird", "black", "blue", "boat", "bold",
 "bolt", "book", "boot", "botany", "bowl", "box", "brain", "brave",
-"bread", "breeze", "breezy", "brew", "bridge", "bright", "bubble", "bug",
-"bunny", "butter",
+"bread", "breeze", "breezy", "brew", "bridge", "brief", "bright", "bubble",
+"bug", "bunny", "bus", "butter",
 "calm", "camera", "canoe", "car", "carrot", "cartoon", "cello", "chair",
 "check", "cheerful", "cheese", "chilly", "chorus", "chummy", "city", "classy",
-"clean", "clever", "clock", "cloudy", "clumsy", "coat", "coconut", "coffee",
-"comet", "content", "cool", "country", "cow", "cream", "crispy", "crown",
-"curly", "curtain",
+"clean", "clerk", "clever", "clock", "cloudy", "clumsy", "coat", "coconut",
+"coffee", "cold", "comet", "content", "cool", "cosmic", "country", "cow",
+"cream", "crispy", "crown", "curly", "curtain",
 "daily", "daisy", "damp", "darken", "data", "deep", "deer", "degree",
 "desert", "desk", "diamond", "dirt", "dizzy", "dog", "doll", "domain",
 "dusty", "duty", "dynamic",
-"eager", "eagle", "earth", "easy", "elated", "elegant", "energy", "engine",
-"epic", "errand", "event", "excited", "exotic",
+"eager", "eagle", "earth", "easy", "either", "elated", "elegant", "empty",
+"energy", "engine", "epic", "errand", "event", "excited", "exotic",
 "famous", "fancy", "farmer", "fast", "fearless", "festive", "field", "fight",
 "finch", "finish", "fire", "fish", "fit", "flower", "fluent", "fluffy",
 "flute", "formal", "free", "fresh", "friend", "fruit", "funny", "furry",
@@ -306,49 +318,50 @@ static const char *wordlist[] =
 "gadfly", "gallon", "gate", "genius", "gentle", "giant", "giddy", "gifted",
 "global", "gold", "goofy", "grain", "grand", "great", "green", "grumpy",
 "guitar", "guppy",
-"hall", "hand", "happy", "harbor", "hat", "hearty", "heavy", "height",
-"helpful", "hill", "hippo", "honest", "honor", "horse", "hotel", "house",
-"hungry", "husky",
+"hall", "hand", "happy", "harbor", "hat", "hearty", "heat", "heavy",
+"height", "helpful", "hill", "hippo", "hockey", "honest", "honor", "horse",
+"hotel", "house", "hungry", "husky",
 "icy", "idea", "imaginary", "immune", "ink", "input", "invent", "iris",
 "island",
 "jacket", "jade", "jagged", "jazz", "jeans", "jet", "jewel", "jog",
 "jolly", "joyous", "judge", "jungle",
-"kayak", "kettle", "key", "kind", "kite", "knight",
-"ladder", "lake", "large", "lawn", "lazy", "leader", "left", "lemon",
-"light", "lion", "little", "lively", "long", "lotus", "loud", "loyal",
-"lucky",
+"kangaroo", "kayak", "kettle", "key", "kind", "kite", "knight", "known",
+"ladder", "lake", "large", "latter", "law", "lawn", "lazy", "leader",
+"left", "lemon", "light", "lion", "little", "lively", "long", "lotus",
+"loud", "loyal", "lucky",
 "magical", "major", "mango", "manic", "marble", "market", "mellow", "melodic",
 "mesa", "mighty", "mint", "mirror", "misty", "mobile", "modern", "month",
 "moon", "most", "museum",
 "narrow", "nature", "nest", "new", "nifty", "noble", "noisy", "north",
 "oasis", "object", "oboe", "ocean", "octopus", "odd", "olive", "onion",
 "orange", "owl", "oxygen",
-"palm", "panda", "pant", "parade", "park", "pastel", "patron", "pear",
-"pencil", "perfect", "phobic", "phoenix", "phone", "piano", "pink", "plain",
-"plane", "planet", "pledge", "plum", "pocket", "polite", "pond", "poodle",
-"potato", "prairie", "praise", "precious", "prose", "proud", "puppy", "purple",
+"palm", "panda", "pant", "parade", "park", "pastel", "patron", "peach",
+"pear", "pencil", "perfect", "phobic", "phoenix", "phone", "piano", "pink",
+"plain", "plane", "planet", "pledge", "plum", "pocket", "polite", "pond",
+"poodle", "potato", "prairie", "praise", "precious", "prose", "proud", "puppy",
+"purple",
 "quail", "quaint", "quick", "quiet", "quote",
-"rabbit", "raccoon", "raft", "rain", "rainy", "rapid", "raven", "reason",
-"red", "remedy", "review", "reward", "rich", "river", "road", "robin",
-"rock", "rocket", "rocky", "role", "rose", "rosebud", "round", "royal",
-"runner", "rustic",
-"safety", "salute", "scary", "scout", "sea", "seed", "shark", "sheep",
-"shelf", "shiny", "ship", "shirt", "shoe", "short", "shrub", "silent",
-"silky", "silly", "silver", "skates", "sky", "sled", "sleepy", "slow",
-"small", "smart", "smiley", "smiling", "smooth", "snail", "snake", "soccer",
-"socks", "space", "spark", "sparrow", "spider", "spirit", "spoon", "square",
-"squash", "squirrel", "stable", "star", "state", "statue", "stealth", "still",
-"stove", "straw", "street", "strong", "studio", "summit", "sun", "sunny",
-"super", "sweet", "swift",
+"rabbit", "raccoon", "raft", "rain", "rainy", "rapid", "raven", "ready", 
+"reason", "red", "remedy", "review", "reward", "rich", "river", "road",
+"robin", "rock", "rocket", "rocky", "role", "rose", "rosebud", "round",
+"royal", "runner", "rustic",
+"safety", "salute", "scary", "scout", "sea", "sector", "seed", "shark",
+"sheep", "shelf", "shiny", "ship", "shirt", "shoe", "short", "shrub",
+"silent", "silky", "silly", "silver", "skates", "sky", "sled", "sleepy",
+"slow", "small", "smart", "smiley", "smiling", "smooth", "snail", "snake",
+"soccer", "socks", "space", "spark", "sparrow", "spider", "spirit", "spoon",
+"square", "squash", "squirrel", "stable", "star", "state", "statue", "stealth",
+"still", "stove", "straw", "street", "strong", "studio", "study", "summit",
+"sun", "sunny", "super", "sweet", "swift",
 "table", "tablet", "tall", "teal", "teapot", "tent", "terrific", "theory",
 "think", "thirsty", "tiger", "timber", "tiny", "tomato", "topic", "town",
 "trail", "train", "tree", "truck", "trumpet", "tuba", "tulip", "turkey",
-"turtle",
+"turtle", "two",
 "uneven", "unicorn", "union", "unit", "unite", "unusual", "urban", "useful",
 "valley", "vanilla", "vase", "vast", "verse", "violet", "violin", "voyage",
-"wagon", "walnut", "warm", "watch", "water", "way", "watery", "wealth",
-"west", "whale", "wide", "wild", "wind", "window", "windy", "witty",
-"wolf", "wonderful", "wooden", "world", "writer",
+"wagon", "walnut", "warm", "watch", "water", "watery", "way", "wealth",
+"west", "whale", "wide", "wild", "wind", "window", "windy", "wine",
+"witty", "wolf", "wonderful", "wooden", "world", "writer",
 "yacht", "yard", "year", "yellow", "young",
 "zany", "zeal", "zebra", "zone", "zoo"
 };
@@ -357,7 +370,6 @@ for(ca = 0; ca < (sizeof(wordlist) / sizeof(char *)); ca++)
 	{
 	for(cs = 0; cs < (sizeof(wordlist) / sizeof(char *)); cs++)
 		{
-		if(ca == cs) continue;
 		for (cn = 0; cn < 1000; cn++)
 			{
 			snprintf(pskstring, PSKSTRING_LEN_MAX, "%s%s%03d", wordlist[ca], wordlist[cs], cn);
@@ -369,7 +381,6 @@ for(ca = 0; ca < (sizeof(wordlist) / sizeof(char *)); ca++)
 	{
 	for(cs = 0; cs < (sizeof(wordlist) / sizeof(char *)); cs++)
 		{
-		if(ca == cs) continue;
 		for (cn = 0; cn < 100; cn++)
 			{
 			snprintf(pskstring, PSKSTRING_LEN_MAX, "%s%s%02d", wordlist[ca], wordlist[cs], cn);
@@ -381,7 +392,6 @@ for(ca = 0; ca < (sizeof(wordlist) / sizeof(char *)); ca++)
 	{
 	for(cs = 0; cs < (sizeof(wordlist) / sizeof(char *)); cs++)
 		{
-		if(ca == cs) continue;
 		for (cn = 0; cn < 10; cn++)
 			{
 			snprintf(pskstring, PSKSTRING_LEN_MAX, "%s%s%d", wordlist[ca], wordlist[cs], cn);
@@ -393,7 +403,6 @@ for(ca = 0; ca < (sizeof(wordlist) / sizeof(char *)); ca++)
 	{
 	for(cs = 0; cs < (sizeof(wordlist) / sizeof(char *)); cs++)
 		{
-		if(ca == cs) continue;
 		snprintf(pskstring, PSKSTRING_LEN_MAX, "%s%s", wordlist[ca], wordlist[cs]);
 		fprintf(fhout,"%s\n", pskstring);
 		}
@@ -709,7 +718,7 @@ return;
 static void keywriteweakpass(FILE *fhout)
 {
 static size_t w;
-static int y;
+static unsigned int y;
 
 static char pskstring[PSKSTRING_LEN_MAX] = {};
 static const char *weakword[] =
@@ -786,7 +795,7 @@ return;
 /*===========================================================================*/
 static void keywriteeudate(FILE *fhout)
 {
-static int d ,m ,y;
+static unsigned int d, m, y;
 
 for(y = 1900; y <= thisyear; y++)
 	{
@@ -843,7 +852,7 @@ return;
 /*===========================================================================*/
 static void keywriteusdate(FILE *fhout)
 {
-static int d ,m ,y;
+static unsigned int d, m, y;
 
 for(y = 1900; y <= thisyear; y++)
 	{
@@ -900,7 +909,7 @@ return;
 /*===========================================================================*/
 static void keywriteyearyear(FILE *fhout)
 {
-static int y, y2, y3;
+static unsigned int y, y2, y3;
 static char pskstring[PSKSTRING_LEN_MAX] = {};
 
 for(y = 1900; y <= thisyear; y++)
@@ -928,45 +937,46 @@ return;
 /*===========================================================================*/
 static void keywriteegn(FILE *fhout)
 {
-static int y, m, d, mc, i, j, c;
+static unsigned int y, m, d, mc, i, j, c;
 static char pskstring[PSKSTRING_LEN_MAX] = {};
-static int w[] = {2, 4, 8, 5, 10, 9, 7, 3, 6};
+static unsigned int w[] = {2, 4, 8, 5, 10, 9, 7, 3, 6};
 
 for(y = 1950; y <= thisyear; y++)
-    {
-    if (y < 2000) mc = 0; else mc = 40;
-    for(m = 1; m <= 12; m++)
-        {
-        for(d = 1; d <= 31; d++)
-	        {
-	        if (m == 2)
-	        {
-	        if ((((y % 4 == 0) && (y % 100 != 0)) || (y % 400 == 0)))
-	            {
-	            if (d > 29) continue;
-	            } else {
-	            if (d > 28) continue;
-	            }
-	        }
-	        if ((m == 4 || m == 6 || m == 9 || m == 11) && d > 30) continue;
-
-	        for (i = 0; i < 1000; i++)
-	            {
-	            snprintf(pskstring, PSKSTRING_LEN_MAX, "%02d%02d%02d%03d", y % 100, m + mc, d, i);
-	            c = 0;
-	            for (j = 0; j < 10; j++)
-	                {
-	                c += (pskstring[j] - 48) * w[j];
-	                }
-	            c %= 11;
-	            if (c == 10) c = 0;
-                pskstring[9] = c + 48;
-                pskstring[10] = 0;
-	            writepsk(fhout, pskstring);
-	            }
-	        }
-	    }
-    }
+	{
+	if (y < 2000) mc = 0; else mc = 40;
+	for(m = 1; m <= 12; m++)
+		{
+		for(d = 1; d <= 31; d++)
+			{
+			if (m == 2)
+				{
+				if ((((y % 4 == 0) && (y % 100 != 0)) || (y % 400 == 0)))
+					{
+					if (d > 29) continue;
+					}
+				else
+					{
+					if (d > 28) continue;
+					}
+				}
+			if ((m == 4 || m == 6 || m == 9 || m == 11) && d > 30) continue;
+			for (i = 0; i < 1000; i++)
+				{
+				snprintf(pskstring, PSKSTRING_LEN_MAX, "%02d%02d%02d%03d", y % 100, m + mc, d, i);
+				c = 0;
+				for (j = 0; j < 9; j++)
+					{
+					c += (pskstring[j] - 48) * w[j];
+					}
+				c %= 11;
+				if (c == 10) c = 0;
+				pskstring[9] = c + 48;
+				pskstring[10] = 0;
+				writepsk(fhout, pskstring);
+				}
+			}
+		}
+	}
 return;
 }
 /*===========================================================================*/
@@ -1020,7 +1030,7 @@ return;
 /*===========================================================================*/
 static void writeessidadd(FILE *fhout, char *essid)
 {
-static int c, d;
+static unsigned int c, d;
 static char essidstring[PSKSTRING_LEN_MAX +PSKSTRING_LEN_MAX +PSKSTRING_LEN_MAX] = {};
 
 for(c = 22222; c <= 99999; c += 11111)
@@ -1448,6 +1458,29 @@ for(k1 = 0; k1 < 0x10000; k1++) fprintf(fhout, "CG3000DV2%04X%c%c\n", k1, essid[
 return;
 }
 /*===========================================================================*/
+static void testcpsrf(FILE *fhout, uint8_t essidlen, uint8_t *essid)
+{
+static int k1;
+static char *cpcrf1 = "CoolpadSurf_";
+static char *cpcrf2 = "Coolpad Surf ";
+
+if(essidlen < 16) return;
+if(memcmp(essid, cpcrf1, 12) == 0)
+	{
+	if((!isdigit(essid[12])) || (!isdigit(essid[13])) || (!isdigit(essid[14])) || (!isdigit(essid[15]))) return;
+	for(k1 = 0; k1 < 10000; k1++) fprintf(fhout, "%04d%.*s\n", k1, 4, &essid[12]);
+	return;
+	}
+if(essidlen < 17) return;
+if(memcmp(essid, cpcrf2, 13) == 0)
+	{
+	if((!isdigit(essid[13])) || (!isdigit(essid[14])) || (!isdigit(essid[15])) || (!isdigit(essid[16]))) return;
+	for(k1 = 0; k1 < 10000; k1++) fprintf(fhout, "%04d%.*s\n", k1, 4, &essid[13]);
+	return;
+	}
+return;
+}
+/*===========================================================================*/
 static void testeasybox(FILE *fhout, uint8_t essidlen, uint8_t *essid)
 {
 static int b;
@@ -1680,7 +1713,7 @@ return;
 static void testrtk(FILE *fhout, uint8_t essidlen, uint8_t *essid)
 {
 static int k1;
-static const char *rtk =  "RTK-";
+static const char *rtk = "RTK-";
 
 if(essidlen == 10)
 	{
@@ -1706,8 +1739,8 @@ return;
 static void testtechnicolor(FILE *fhout, uint8_t essidlen, uint8_t *essid)
 {
 static int k1;
-static const char *tc8715d =  "TC8715D";
-static const char *tc8717t =  "TC8717T";
+static const char *tc8715d = "TC8715D";
+static const char *tc8717t = "TC8717T";
 
 if(essidlen >= 9)
 	{
@@ -1842,12 +1875,25 @@ for(k2 = 0; k2 < 10; k2++)
 return;
 }
 /*===========================================================================*/
+static void testx2g(FILE *fhout, uint8_t essidlen, uint8_t *essid)
+{
+static int k;
+static char *x2g = "_2G";
+
+if(essidlen < 9) return;
+if(memcmp(&essid[essidlen -9], x2g, 2) != 0) return;
+if((!isdigit(essid[essidlen -1])) || (!isdigit(essid[essidlen -2])) || (!isdigit(essid[essidlen -3])) || (!isdigit(essid[essidlen -4])) || (!isdigit(essid[essidlen -5])) || (!isdigit(essid[essidlen -6]))) return;
+for(k = 0; k < 0x100; k++) fprintf(fhout, "%02X%s\n", k, &essid[essidlen -6]);
+return;
+}
+/*===========================================================================*/
 static void prepareessid(FILE *fhout, uint8_t essidlen, uint8_t *essid)
 {
 static int pi, po;
 static char essidtmp[PSKSTRING_LEN_MAX] = {};
 
 if((essidlen == 0) || (essidlen > 32)) return;
+testx2g(fhout, essidlen, essid);
 testairtel(fhout, essidlen, essid);
 testalcatellinkzone(fhout, essidlen, essid);
 testarrisizzi(fhout, essidlen, essid);
@@ -1856,6 +1902,7 @@ testattwifi(fhout, essidlen, essid);
 testaxtelxtremo(fhout, essidlen, essid);
 testcabovisao(fhout, essidlen, essid);
 testcg3000dv2(fhout, essidlen, essid);
+testcpsrf(fhout, essidlen, essid);
 testeasybox(fhout, essidlen, essid);
 testglocal(fhout, essidlen, essid);
 testhotbox(fhout, essidlen, essid);
@@ -1871,6 +1918,7 @@ testukrtelecom(fhout, essidlen, essid);
 testwifirsu(fhout, essidlen, essid);
 testwlan(fhout, essidlen, essid);
 
+if(noessidcombinationflag == true) return;
 writeessidsweeped(fhout, essidlen, essid);
 po = 0;
 memset(&essidtmp, 0, PSKSTRING_LEN_MAX);
@@ -2019,7 +2067,7 @@ snprintf(pskstring, PSKSTRING_LEN_MAX, "2%012llx", macaddr);
 writepsk(fhout, pskstring);
 snprintf(pskstring, PSKSTRING_LEN_MAX, "m%012llx", macaddr);
 writepsk(fhout, pskstring);
-snprintf(pskstring, PSKSTRING_LEN_MAX, "555A5053%08llx", macaddr &0xffffffff);
+snprintf(pskstring, PSKSTRING_LEN_MAX, "555A5053%08llX", macaddr &0xffffffff);
 writepsk(fhout, pskstring);
 snprintf(pskstring, PSKSTRING_LEN_MAX, "8747%06llx", macaddr &0xffffff);
 writepsk(fhout, pskstring);
@@ -2078,15 +2126,17 @@ static int c;
 static unsigned long long int oui;
 static unsigned long long int nic;
 static int swap;
-
+static int me;
 static char pskstring[PSKSTRING_LEN_MAX] = {};
 
+me = macaddr &0xffffff;
+fprintf(fhout, "05%6d\n", me);
 oui = macaddr &0xffffff000000L;
-nic = (macaddr &0xffffffL) -8;
-for(c = 0; c < 0x10; c++) writebssid(fhout, oui +nic +c);
+nic = (macaddr -0x10) &0xffffffL;
+for(c = 0; c <= 0x10; c++) writebssid(fhout, oui +((nic +c) &0xffffffL));
 swap = (nic >> 8) & 0xffff;
 	{
-	swap = (swap & 0xf000) >> 12 | (swap & 0x0f00) >> 4 | (swap & 0x00f0) << 4 |  (swap & 0x000f) << 12;
+	swap = (swap & 0xf000) >> 12 | (swap & 0x0f00) >> 4 | (swap & 0x00f0) << 4 | (swap & 0x000f) << 12;
 	snprintf(pskstring, PSKSTRING_LEN_MAX, "000000%04X", swap);
 	fprintf(fhout, "%s\n", pskstring);
 	}
@@ -2236,7 +2286,6 @@ while(1)
 		aktread++;
 		continue;
 		}
-
 	if(((linein[32] != ':') && (linein[45] != ':') && (linein[58] != ':')) && ((linein[32] != '*') && (linein[45] != '*') && (linein[58] != '*')))
 		{
 		fprintf(stderr, "reading hash line %d failed: %s\n", aktread, linein);
@@ -2300,7 +2349,6 @@ while(1)
 		aktread++;
 		continue;
 		}
-
 	if((linein[3] != '*') && (linein[6] != '*') && (linein[39] != '*') && (linein[52] != '*') && (linein[65] != '*'))
 		{
 		fprintf(stderr, "reading hash line %d failed: %s\n", aktread, linein);
@@ -2340,10 +2388,10 @@ static int getwpapskfmt(int lenlinein, char *linein)
 static int p;
 static const char *johnformat = "$WPAPSK$";
 
-	for(p = 0; p < lenlinein -8; p++)
-		{
-		if(memcmp(&linein[p], johnformat, 8) == 0) return p;
-		}
+for(p = 0; p < lenlinein -8; p++)
+	{
+	if(memcmp(&linein[p], johnformat, 8) == 0) return p;
+	}
 return 0;
 }
 /*===========================================================================*/
@@ -2476,7 +2524,6 @@ if(macapname != NULL)
 	macaddr = strtoull(macapname, &macaddrstop, 16);
 	if((macaddrstop -macapname) != 12) fprintf(stderr, "invalid MAC specified\n");
 	}
-
 memset(&essid, 0, ESSID_LEN_MAX);
 essidlen = strlen(essidname);
 if(essidname != NULL)
@@ -2522,30 +2569,27 @@ printf("%s %s (C) %s ZeroBeat\n"
 	"-h          : show this help\n"
 	"-v          : show version\n"
 	"\n"
-	"--netgear     : include weak NETGEAR / ORBI / NTGR_VMB candidates\n"
-	"--askeyarris  : include weak MySpectrumWiFI / SpectrumSetup / MyCharterWiFI candidates\n"
-	"                list will be > 3GB\n"
-	"--digit10     : include weak 10 digit candidates (INFINITUM, ALHN, INEA, VodafoneNet, VIVACOM)\n"
-	"                list will be > 1GB\n"
-	"--phome       : include weak PEGATRON HOME candidates\n"
-	"--tenda       : include weak TENDA candidates\n"
-	"--ee          : include weak EE BrightBox candidates\n"
-	"                list will be > 3GB\n"
-	"--weakpass    : include weak password candidates\n"
-	"--eudate      : include complete european dates\n"
-	"--usdate      : include complete american dates\n"
-	"--wpskeys     : include complete WPS keys\n"
-	"--egn         : include Bulgarian EGN\n"
-	"--help        : show this help\n"
-	"--version     : show version\n"
+	"--maconly           : print only candidates based on ACCESS POINT MAC\n"
+	"--noessidcombination: exclude ESSID combinations\n"
+	"--netgear           : include weak NETGEAR / ORBI / NTGR_VMB candidates\n"
+	"--askeyarris        : include weak MySpectrumWiFI / SpectrumSetup / MyCharterWiFI candidates\n"
+	"                      list will be > 3.2GB\n"
+	"--digit10           : include weak 10 digit candidates (INFINITUM, ALHN, INEA, VodafoneNet, VIVACOM)\n"
+	"                      list will be > 1GB\n"
+	"--phome             : include weak PEGATRON HOME candidates\n"
+	"--tenda             : include weak TENDA candidates\n"
+	"--ee                : include weak EE BrightBox candidates\n"
+	"                      list will be > 3GB\n"
+	"--weakpass          : include weak password candidates\n"
+	"--eudate            : include complete european dates\n"
+	"--usdate            : include complete american dates\n"
+	"--wpskeys           : include complete WPS keys\n"
+	"--egn               : include Bulgarian EGN\n"
+	"--help              : show this help\n"
+	"--version           : show version\n"
 	"\n"
-	"if hcxpsktool recovered your password, you should change it immediately!\n"
-	"\n"
-	"examples:\n"
-	"hcxpsktool -i hashfile.hccapx | sort | uniq | hashcat -m 2500 hashfile.hccapx\n"
-	"hcxpsktool -z hashfile.16800 | sort | uniq | hashcat -m 16800 hashfile.16800\n"
-	"hcxpsktool -z hashfile.16800 | sort | uniq | john --stdin --format=wpapsk-opencl hashfile.16800\n"
-	"\n", eigenname, VERSION_TAG, VERSION_YEAR, eigenname);
+	"if hcxpsktool recovered your password, you should change it immediately!\n\n",
+	eigenname, VERSION_TAG, VERSION_YEAR, eigenname);
 exit(EXIT_SUCCESS);
 }
 /*---------------------------------------------------------------------------*/
@@ -2579,6 +2623,8 @@ eeflag = false;
 egnflag = false;
 eudateflag = false;
 hb5flag = false;
+maconlyflag = false;
+noessidcombinationflag = false;
 netgearflag = false;
 phomeflag = false;
 podaflag = false;
@@ -2591,6 +2637,8 @@ wpskeysflag = false;
 static const char *short_options = "c:i:j:z:o:e:b:o:hv";
 static const struct option long_options[] =
 {
+	{"maconly",			no_argument,		NULL,	HCXD_MACONLY},
+	{"noessidcombination",		no_argument,		NULL,	HCXD_NOESSIDCOMBINATION},
 	{"netgear",			no_argument,		NULL,	HCXD_NETGEAR},
 	{"askeyarris",			no_argument,		NULL,	HCXD_ASKEYARRIS},
 	{"digit10",			no_argument,		NULL,	HCXD_DIGIT10},
@@ -2615,10 +2663,18 @@ while((auswahl = getopt_long (argc, argv, short_options, long_options, &index))
 	{
 	switch (auswahl)
 		{
+		case HCXD_MACONLY:
+		maconlyflag = true;
+		break;
+
 		case HCXD_NETGEAR:
 		netgearflag = true;
 		break;
 
+		case HCXD_NOESSIDCOMBINATION:
+		noessidcombinationflag = true;
+		break;
+
 		case HCXD_ASKEYARRIS:
 		askeyarrisflag = true;
 		break;
@@ -2752,16 +2808,22 @@ if(pskname != NULL)
 		exit(EXIT_FAILURE);
 		}
 	processbssids(fhpsk);
-	processessids(fhpsk);
-	processbssidsessids(fhpsk);
-	processadditionals(fhpsk);
+	if(maconlyflag == false)
+		{
+		processessids(fhpsk);
+		processbssidsessids(fhpsk);
+		processadditionals(fhpsk);
+		}
 	}
 else
 	{
 	processbssids(stdout);
-	processessids(stdout);
-	processbssidsessids(stdout);
-	processadditionals(stdout);
+	if(maconlyflag == false)
+		{
+		processessids(stdout);
+		processbssidsessids(stdout);
+		processadditionals(stdout);
+		}
 	}
 
 
