@@ -12,6 +12,11 @@
 #define PMKIDEAPOL_BUFFER_LEN	1024
 #define HASHLIST_MAX		50000
 
+#define PBKDF2_LINE_LEN		1024
+#define PMK_LEN			32
+#define PSK_LEN_MIN		8
+#define PSK_LEN_MAX		63
+
 #define HCX_HASH_TYPE			1
 #define HCX_HASH_MIN			2
 #define HCX_HASH_MAX			3
@@ -28,25 +33,32 @@
 #define HCX_FILTER_MAC_AP		14
 #define HCX_FILTER_MAC_CLIENT		15
 #define HCX_FILTER_MAC_LIST_IN		16
-#define HCX_FILTER_VENDOR		17
-#define HCX_FILTER_ESSID		18
-#define HCX_FILTER_ESSID_PART		19
-#define HCX_FILTER_RC			20
-#define HCX_FILTER_M12			21
-#define HCX_FILTER_M1234		22
-#define HCX_FILTER_M1M2ROGUE		23
-#define HCX_PSK				24
-#define HCX_PMK				25
-#define HCX_VENDOR_OUT			26
-#define HCX_INFO_OUT			27
-#define HCX_HCCAPX_OUT			28
-#define HCX_HCCAP_OUT			29
-#define HCX_HCCAP_SINGLE_OUT		30
-#define HCX_JOHN_OUT			31
+#define HCX_FILTER_MAC_LIST_SKIP	17
+#define HCX_FILTER_VENDOR		18
+#define HCX_FILTER_VENDOR_AP		19
+#define HCX_FILTER_VENDOR_CLIENT	20
+#define HCX_FILTER_ESSID		21
+#define HCX_FILTER_ESSID_PART		22
+#define HCX_FILTER_RC			23
+#define HCX_FILTER_M12			24
+#define HCX_FILTER_M1234		25
+#define HCX_FILTER_M1M2ROGUE		26
+#define HCX_PSK				27
+#define HCX_PMK				28
+#define HCX_VENDOR_OUT			29
+#define HCX_INFO_OUT			30
+#define HCX_INFO_VENDOR_OUT		31
+#define HCX_INFO_VENDOR_AP_OUT		32
+#define HCX_INFO_VENDOR_CLIENT_OUT	33
+#define HCX_HCCAPX_OUT			34
+#define HCX_HCCAP_OUT			35
+#define HCX_HCCAP_SINGLE_OUT		36
+#define HCX_JOHN_OUT			37
 #define HCX_PMKIDEAPOL_IN		'i'
 #define HCX_PMKIDEAPOL_OUT		'o'
 #define HCX_ESSID_OUT			'E'
 #define HCX_DOWNLOAD_OUI		'd'
+#define HCX_PBKDF2_IN			'p'
 #define HCX_HELP			'h'
 #define HCX_VERSION			'v'
 /*===========================================================================*/
@@ -54,6 +66,7 @@
 struct hashlist_s
 {
  uint8_t		type;
+#define HS_REMOVED	0xff;
 #define HS_PMKID	1;
 #define HS_EAPOL	2;
  uint8_t		hash[16];
@@ -69,7 +82,7 @@ struct hashlist_s
 typedef struct hashlist_s hashlist_t;
 #define	HASHLIST_SIZE (sizeof(hashlist_t))
 
-static int sort_maclist_by_essid(const void *a, const void *b)
+static int sort_hashlist_by_essid(const void *a, const void *b)
 {
 const hashlist_t *ia = (const hashlist_t *)a;
 const hashlist_t *ib = (const hashlist_t *)b;
@@ -79,7 +92,7 @@ else if(memcmp(ia->essid, ib->essid, ESSID_LEN_MAX) < 0) return -1;
 return 0;
 }
 
-static int sort_maclist_by_essidlen(const void *a, const void *b)
+static int sort_hashlist_by_essidlen(const void *a, const void *b)
 {
 const hashlist_t *ia = (const hashlist_t *)a;
 const hashlist_t *ib = (const hashlist_t *)b;
@@ -91,7 +104,7 @@ else if(memcmp(ia->essid, ib->essid, ia->essidlen) < 0) return -1;
 return 0;
 }
 
-static int sort_maclist_by_macap(const void *a, const void *b)
+static int sort_hashlist_by_macap(const void *a, const void *b)
 {
 const hashlist_t *ia = (const hashlist_t *)a;
 const hashlist_t *ib = (const hashlist_t *)b;
@@ -101,7 +114,7 @@ else if(memcmp(ia->ap, ib->ap, 6) < 0) return -1;
 return 0;
 }
 
-static int sort_maclist_by_macclient(const void *a, const void *b)
+static int sort_hashlist_by_macclient(const void *a, const void *b)
 {
 const hashlist_t *ia = (const hashlist_t *)a;
 const hashlist_t *ib = (const hashlist_t *)b;
@@ -110,9 +123,22 @@ if(memcmp(ia->client, ib->client, 6) > 0) return 1;
 else if(memcmp(ia->client, ib->client, 6) < 0) return -1;
 return 0;
 }
+
+static int sort_hashlist_by_type(const void *a, const void *b)
+{
+const hashlist_t *ia = (const hashlist_t *)a;
+const hashlist_t *ib = (const hashlist_t *)b;
+
+if(ia->type > ib->type) return 1;
+else if(ia->type < ib->type) return -1;
+return 0;
+}
 /*===========================================================================*/
 struct ouilist_s
 {
+ uint8_t		type;
+#define TYPE_AP		1
+#define TYPE_CLIENT	2
  uint8_t		oui[3];
 #define VENDOR_LEN_MAX	128
  char			vendor[VENDOR_LEN_MAX];
@@ -167,7 +193,15 @@ else if(memcmp(ia->mac, ib->mac, 6) < 0) return -1;
 return 0;
 }
 /*===========================================================================*/
-
-
-
-
+struct pmklist_s
+{
+ uint8_t	status;
+ uint8_t	essidlen;
+ uint8_t	psklen;
+ uint8_t	pmk[PMK_LEN];
+ uint8_t	essid[ESSID_LEN_MAX];
+ uint8_t	psk[PSK_LEN_MAX];
+};
+typedef struct pmklist_s pmklist_t;
+#define	PMKLIST_SIZE (sizeof(pmklist_t))
+/*===========================================================================*/
