@@ -4,17 +4,19 @@
 #define JOHN_LINE_LEN 1024
 #define PSKSTRING_LEN_MAX 64
 
-#define HCXD_NETGEAR			1
-#define HCXD_ASKEYARRIS			2
-#define HCXD_PHOME			3
-#define HCXD_TENDA			4
-#define HCXD_EE			5
-#define HCXD_WEAKPASS			6
-#define HCXD_EUDATE			7
-#define HCXD_USDATE			8
-#define HCXD_WPSKEYS			9
-#define HCXD_DIGIT10			10
-#define HCXD_EGN			11
+#define HCXD_MACONLY			1
+#define HCXD_NOESSIDCOMBINATION		2
+#define HCXD_NETGEAR			3
+#define HCXD_ASKEYARRIS			4
+#define HCXD_PHOME			5
+#define HCXD_TENDA			6
+#define HCXD_EE				7
+#define HCXD_WEAKPASS			8
+#define HCXD_EUDATE			9
+#define HCXD_USDATE			10
+#define HCXD_WPSKEYS			11
+#define HCXD_DIGIT10			12
+#define HCXD_EGN			13
 #define HCXD_HELP			'h'
 #define HCXD_VERSION			'v'
 
