@@ -17,11 +17,23 @@ bool isasciistring(int len, uint8_t *buffer)
 uint8_t p;
 for(p = 0; p < len; p++)
 	{
+	if(buffer[p] == 0) return true;
 	if((buffer[p] < 0x20) || (buffer[p] > 0x7e) || (buffer[p] == ':')) return false;
 	}
 return true;
 }
 /*===========================================================================*/
+bool isasciistring2(int len, uint8_t *buffer)
+{
+uint8_t p;
+for(p = 0; p < len; p++)
+	{
+	if(buffer[p] == 0) return true;
+	if((buffer[p] < 0x20) || (buffer[p] > 0x7e)) return false;
+	}
+return true;
+}
+/*===========================================================================*/
 bool ishexvalue(const char *str, size_t len)
 {
 size_t c;
