@@ -42,7 +42,7 @@ curl_easy_setopt(hnd, CURLOPT_URL, "http://standards-oui.ieee.org/oui/oui.txt");
 curl_easy_setopt(hnd, CURLOPT_NOPROGRESS, 1L);
 curl_easy_setopt(hnd, CURLOPT_MAXREDIRS, 5L);
 curl_easy_setopt(hnd, CURLOPT_WRITEDATA, fhoui);
-
+curl_easy_setopt(hnd, CURLOPT_NOPROGRESS, 0L);
 ret = curl_easy_perform(hnd);
 curl_easy_cleanup(hnd);
 fclose(fhoui);
@@ -489,6 +489,14 @@ while ((auswahl = getopt(argc, argv, "m:v:p:P:e:x:dh")) != -1)
 				}
 			}
 		optarg[6] = 0;
+		for(p1 = 0; p1 < 6; p1++)
+			{
+			if(!isxdigit(optarg[p1]))
+				{
+				fprintf(stderr, "error wrong oui %s (need eg. 11:22:33:44:55:aa or 112233)\n", optarg);
+				exit(EXIT_FAILURE);
+				}
+			}
 		oui = strtoull(optarg, NULL, 16);
 		mode = 'm';
 		break;
