@@ -18,10 +18,17 @@
 /*===========================================================================*/
 /* globale Konstante */
 
+struct memory
+{
+char *response;
+size_t size;
+};
+
 static long int uploadcountok;
 static long int uploadcountfailed;
 static const char *wpasecurl = "https://wpa-sec.stanev.org";
 static bool removeflag = false;
+struct memory *curlmem;
 /*===========================================================================*/
 static int testwpasec(long int timeout)
 {
@@ -46,6 +53,21 @@ curl_global_cleanup();
 return res;
 }
 /*===========================================================================*/
+static size_t cb(void *data, size_t size, size_t nmemb, void *userp)
+{
+char *ptr;
+size_t realsize = size *nmemb;
+curlmem = (struct memory *)userp;
+ 
+ptr = realloc(curlmem->response, curlmem->size +realsize +1);
+if(ptr == NULL) return 0;
+curlmem->response = ptr;
+memcpy(&(curlmem->response[curlmem->size]), data, realsize);
+curlmem->size += realsize;
+curlmem->response[curlmem->size] = 0;
+return realsize;
+}
+/*===========================================================================*/
 static bool sendcap2wpasec(char *sendcapname, long int timeout, char *keyheader, char *emailheader)
 {
 CURL *curl;
@@ -57,35 +79,38 @@ struct curl_httppost *formpost=NULL;
 struct curl_httppost *lastptr=NULL;
 struct curl_slist *headerlist=NULL;
 static const char buf[] = "Expect:";
+struct memory chunk;
 
 printf("uploading %s to %s\n", sendcapname, wpasecurl);
+memset(&chunk, 0, sizeof(chunk));
 curl_global_init(CURL_GLOBAL_ALL);
 curl_formadd(&formpost, &lastptr, CURLFORM_COPYNAME, "file", CURLFORM_FILE, sendcapname, CURLFORM_END);
-if(emailheader != NULL)
-	{
-	curl_formadd(&formpost, &lastptr, CURLFORM_COPYNAME, "email", CURLFORM_PTRCONTENTS, emailheader, CURLFORM_END);
-	}
+if(emailheader != NULL) curl_formadd(&formpost, &lastptr, CURLFORM_COPYNAME, "email", CURLFORM_PTRCONTENTS, emailheader, CURLFORM_END);
 
 curl = curl_easy_init();
 headerlist = curl_slist_append(headerlist, buf);
 if(curl)
 	{
+	curl_easy_setopt(curl, CURLOPT_NOPROGRESS, 0L);
+	curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, cb);
+	curl_easy_setopt(curl, CURLOPT_WRITEDATA, (void *)&chunk);
 	curl_easy_setopt(curl, CURLOPT_URL, wpasecurl);
 	curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, timeout);
 	curl_easy_setopt(curl, CURLOPT_HTTPPOST, formpost);
-	if(keyheader)
-		{
-		curl_easy_setopt(curl, CURLOPT_COOKIE, keyheader);
-		}
+	if(keyheader) curl_easy_setopt(curl, CURLOPT_COOKIE, keyheader);
 	res = curl_easy_perform(curl);
 	if(res == CURLE_OK)
 		{
-		printf("\n\x1B[32mupload done\x1B[0m\n\n");
+		printf("upload done\n");
 		if(removeflag == true)
 			{
 			ret = remove(sendcapname);
-			if(ret != 0)
-				fprintf(stderr, "couldn't remove %s\n", sendcapname);
+			if(ret != 0) fprintf(stderr, "couldn't remove %s\n", sendcapname);
+			}
+		if(curlmem->response != NULL)
+			{
+			printf("\n%s\n\n", curlmem->response);
+			free(curlmem->response);
 			}
 		}
 	else
@@ -93,7 +118,6 @@ if(curl)
 		fprintf(stderr, "\n\x1B[31mupload to %s failed: %s\x1B[0m\n\n", wpasecurl, curl_easy_strerror(res));
 		uploadflag = false;
 		}
-
 	curl_easy_cleanup(curl);
 	curl_formfree(formpost);
 	curl_slist_free_all(headerlist);
@@ -203,8 +227,7 @@ while ((auswahl = getopt(argc, argv, "k:u:t:e:Rhv")) != -1)
 		}
 	}
 
-if(testwpasec(timeout) != CURLE_OK)
-	return EXIT_SUCCESS;
+if(testwpasec(timeout) != CURLE_OK) return EXIT_SUCCESS;
 
 for(index = optind; index < argc; index++)
 	{
@@ -212,29 +235,17 @@ for(index = optind; index < argc; index++)
 		{
 		if(sendcap2wpasec(argv[index], timeout, keyheader, emailaddr) == false)
 			{
-			if(sendcap2wpasec(argv[index], 60, keyheader, emailaddr) == true)
-				{
-				uploadcountok++;
-				}
-			else
-				uploadcountfailed++;
+			if(sendcap2wpasec(argv[index], 60, keyheader, emailaddr) == true) uploadcountok++;
+			else uploadcountfailed++;
 			}
-		else
-			uploadcountok++;
+		else uploadcountok++;
 		}
+	else printf("file not found: %s\n", argv[index]);
 	}
 
-if(uploadcountok == 1)
-	printf("\x1B[32m%ld cap uploaded to %s\x1B[0m\n", uploadcountok, wpasecurl);
-
-if(uploadcountok > 1)
-	printf("\x1B[32m%ld caps uploaded to %s\x1B[0m\n", uploadcountok, wpasecurl);
-
-if(uploadcountfailed == 1)
-	printf("\x1B[31m%ld cap failed to upload to %s\x1B[0m\n", uploadcountfailed, wpasecurl);
-
-if(uploadcountfailed > 1)
-	printf("\x1B[31m%ld caps failed to upload to %s\x1B[0m\n", uploadcountfailed, wpasecurl);
-
+if(uploadcountok == 1) printf("\x1B[32m%ld cap uploaded to %s\x1B[0m\n", uploadcountok, wpasecurl);
+if(uploadcountok > 1) printf("\x1B[32m%ld caps uploaded to %s\x1B[0m\n", uploadcountok, wpasecurl);
+if(uploadcountfailed == 1) printf("\x1B[31m%ld cap failed to upload to %s\x1B[0m\n", uploadcountfailed, wpasecurl);
+if(uploadcountfailed > 1) printf("\x1B[31m%ld caps failed to upload to %s\x1B[0m\n", uploadcountfailed, wpasecurl);
 return EXIT_SUCCESS;
 }
